@@ -1,50 +1,39 @@
 import argparse
-import errno
-import math
-import os
 import shlex
 import shutil
-import socket
 import subprocess
-from datetime import datetime, timedelta
 from pathlib import Path
-from random import randrange
 from shutil import which
-from time import sleep
 
 import ffmpeg
-import humanize
 import pandas as pd
-import psutil
 import sqlite_utils
 from catt.api import CattDevice
 from rich.prompt import Confirm
-from tabulate import tabulate
 
 from xklb.db import sqlite_con
 from xklb.utils import (
     CAST_NOW_PLAYING,
     DEFAULT_MPV_SOCKET,
-    FAKE_SUBTITLE,
-    Pclose,
-    Subcommand,
-    audio_exclude_string,
-    audio_include_string,
+    SC,
     cmd,
-    cmd_interactive,
     filter_None,
     flatten,
     get_ip_of_chromecast,
-    get_ordinal_media,
-    human_time,
     log,
+    pkill,
+)
+from xklb.utils_player import (
+    delete_media,
+    get_ordinal_media,
+    listen_chromecast,
+    local_player,
     mark_media_watched,
     mv_to_keep_folder,
-    os_bg_kwargs,
+    printer,
     remove_media,
-    resize_col,
-    tube_exclude_string,
-    tube_include_string,
+    socket_play,
+    watch_chromecast,
 )
 
 idle_mpv = lambda args: ["mpv", "--idle", f"--input-ipc-server={args.mpv_socket}"]
@@ -55,7 +44,7 @@ pd.set_option("display.float_format", lambda x: "%.5f" % x)
 
 
 def override_sort(string):
-    YEAR_MONTH = lambda var: f"cast(strftime('%Y%m',datetime({var} / 1000000000, 'unixepoch')) as int)"
+    YEAR_MONTH = lambda var: f"cast(strftime('%Y%m', datetime({var}, 'unixepoch')) as int)"
 
     return (
         string.replace("month_created", YEAR_MONTH("time_created"))
@@ -65,7 +54,7 @@ def override_sort(string):
     )
 
 
-def parse_args(default_db, default_chromecast=""):
+def parse_args(action, default_db, default_chromecast=""):
     parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
 
     parser.add_argument(
@@ -101,7 +90,6 @@ if you want exact times you can use --where duration=6*60
         "--sort",
         "-u",
         nargs="+",
-        default=["priority"],
         help="""Sort media with SQL expressions
 -u duration means shortest media first
 -u duration desc means longest media first
@@ -137,24 +125,6 @@ Double spaces means one space
     )
     parser.add_argument("--exclude", "-E", "-e", nargs="+", action="extend", default=[])
 
-    parser.add_argument(
-        "--delete",
-        "--remove",
-        "--erase",
-        "--rm",
-        "-rm",
-        action="store_true",
-        help="lb tw -s 'something to delete' -rm",
-    )
-    parser.add_argument(
-        "--mark-watched",
-        "--watched",
-        "--mark_played",
-        "--played",
-        action="store_true",
-        help="lb wt -s 'something to mark seen' --mark-watched",
-    )
-
     parser.add_argument(
         "--chromecast-device",
         "-cast-to",
@@ -193,10 +163,8 @@ Double spaces means one space
 -p a means print an aggregate report
 -p f means print only filenames -- useful for piping to other utilities like xargs or GNU Parallel""",
     )
-    parser.add_argument("--print-column", "-col", nargs="*", help="Include a non-standard column when printing")
-    parser.add_argument(
-        "--limit", "-L", "-l", "-queue", "--queue", default=DEFAULT_PLAY_QUEUE, help="Set play queue size"
-    )
+    parser.add_argument("--cols", "-cols", "-col", nargs="*", help="Include a non-standard column when printing")
+    parser.add_argument("--limit", "-L", "-l", "-queue", "--queue", help="Set play queue size")
     parser.add_argument("--skip", "-S", help="Offset from the top of an ordered query; wt -S10 to skip ten videos")
 
     parser.add_argument(
@@ -231,20 +199,24 @@ Double spaces means one space
     parser.add_argument("--ignore-errors", "--ignoreerrors", action="store_true")
     parser.add_argument("--verbose", "-v", action="count", default=0)
     args = parser.parse_args()
+    args.action = action
 
     if args.db:
         args.database = args.db
 
-    if args.limit == DEFAULT_PLAY_QUEUE and args.print:
-        args.limit = None
+    if not args.limit and all([not args.print, args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]]):
+        args.limit = DEFAULT_PLAY_QUEUE
     elif args.limit in ["inf", "all"]:
         args.limit = None
 
+    if not args.sort and args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
+        args.sort = ["priority"]
     if args.sort:
         args.sort = " ".join(args.sort)
+        args.sort = override_sort(args.sort)
 
-    if args.print_column:
-        args.print_column = list(flatten([s.split(",") for s in args.print_column]))
+    if args.cols:
+        args.cols = list(flatten([s.split(",") for s in args.cols]))
 
     if args.duration:
         SEC_TO_M = 60
@@ -286,9 +258,6 @@ Double spaces means one space
 
         args.size = size_rules
 
-    if args.sort:
-        args.sort = override_sort(args.sort)
-
     if args.chromecast:
         args.cc = CattDevice(args.chromecast_device, lazy=True)
         args.cc_ip = get_ip_of_chromecast(args.chromecast_device)
@@ -318,19 +287,8 @@ def transcode(next_video):
     return next_video
 
 
-def delete_media(args, media_file: str):
-    if len(args.prefix) > 0:
-        Path(media_file).unlink()
-    elif which("trash-put") is not None:
-        cmd("trash-put", media_file, strict=False)
-    else:
-        Path(media_file).unlink()
-
-    remove_media(args, media_file, quiet=True)
-
-
 def post_act(args, media_file: str):
-    if args.action in [Subcommand.listen, Subcommand.watch, Subcommand.tubelisten, Subcommand.tubewatch]:
+    if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
         mark_media_watched(args, media_file)
 
     if args.post_action == "keep":
@@ -374,152 +332,10 @@ def externalize_subtitle(media_file):
     return subtitles_file
 
 
-def watch_chromecast(args, m):
-    subtitles_file = externalize_subtitle(m["path"])
-
-    if "vlc" in args.player:
-        catt_log = cmd(
-            "vlc",
-            "--sout",
-            "#chromecast",
-            f"--sout-chromecast-ip={args.cc_ip}",
-            "--demux-filter=demux_chromecast",
-            "--sub-file=" + subtitles_file if subtitles_file else "",
-            *args.player[1:],
-            m["path"],
-        )
-    else:
-        if args.action in [Subcommand.watch, Subcommand.listen]:
-            catt_log = cmd(
-                "catt",
-                "-d",
-                args.chromecast_device,
-                "cast",
-                "-s",
-                subtitles_file if subtitles_file else FAKE_SUBTITLE,
-                m["path"],
-            )
-        else:
-            catt_log = args.cc.play_url(m["path"], resolve=True, block=True)
-
-    if subtitles_file:
-        Path(subtitles_file).unlink(missing_ok=True)
-    return catt_log
-
-
-def listen_chromecast(args, m):
-    Path(CAST_NOW_PLAYING).write_text(m["path"])
-    Path(FAKE_SUBTITLE).touch()
-    if not args.with_local:
-        if args.action in [Subcommand.watch, Subcommand.listen]:
-            catt_log = cmd("catt", "-d", args.chromecast_device, "cast", "-s", FAKE_SUBTITLE, m["path"])
-        else:
-            catt_log = args.cc.play_url(m["path"], resolve=True, block=True)
-    else:
-        cast_process = subprocess.Popen(
-            ["catt", "-d", args.chromecast_device, "cast", "-s", FAKE_SUBTITLE, m["path"]], **os_bg_kwargs()
-        )
-        sleep(0.974)  # imperfect lazy sync; I use keyboard shortcuts to send `set speed` commands to mpv for resync
-        # if pyChromecast provides a way to sync accurately that would be very interesting to know; I have not researched it
-        cmd_interactive(*args.player, "--", m["path"])
-        catt_log = Pclose(cast_process)  # wait for chromecast to stop (you can tell any chromecast to pause)
-        sleep(3.0)  # give chromecast some time to breathe
-
-    return catt_log
-
-
-def calculate_duration(args, m):
-    start = 0
-    end = m.duration
-
-    if args.start:
-        if args.start == "wadsworth":
-            start = m.duration * 0.3
-        else:
-            start = args.start
-    if args.end:
-        if args.end == "dawsworth":
-            end = m.duration * 0.65
-        elif "+" in args.end:
-            end = (m.duration * 0.3) + args.end
-        else:
-            end = args.end
-
-    return start, end
-
-
-def socket_play(args, m):
-    if args.sock is None:
-        subprocess.Popen(["mpv", "--idle", "--input-ipc-server=" + args.mpv_socket])
-        while not os.path.exists(args.mpv_socket):
-            sleep(0.2)
-        args.sock = socket.socket(socket.AF_UNIX)
-        args.sock.connect(args.mpv_socket)
-
-    start, end = calculate_duration(args, m)
-
-    try:
-        bias_to_acts = m.index + 1 / args.limit
-        start = randrange(int(start * bias_to_acts), int(end - args.interdimensional_cable + 1))
-        end = start + args.interdimensional_cable
-    except:
-        pass
-    if end == 0:
-        return
-
-    play_opts = f"start={start},save-position-on-quit=no"
-    if args.action in [Subcommand.listen, Subcommand.tubelisten]:
-        play_opts += ",video=no,really-quiet=yes"
-    elif args.action in [Subcommand.watch, Subcommand.tubewatch]:
-        play_opts += ",fullscreen=yes,force-window=yes,really-quiet=yes"
-
-    if args.action in [Subcommand.tubelisten, Subcommand.tubewatch]:
-        play_opts += ",script-opts=ytdl_hook-try_ytdl_first=yes"
-
-    f = m.path.replace("\\", "\\\\").replace('"', '\\"').replace("\n", "\\n")
-    args.sock.send((f'raw loadfile "{f}" replace "{play_opts}" \n').encode("utf-8"))
-    sleep(args.interdimensional_cable)
-
-
-def local_player(args, m, media_file):
-    if args.player:
-        player = args.player
-    elif which("mpv") is None:
-        player = ["xdg-open"]
-    else:
-        player = [which("mpv")]
-        if args.action in [Subcommand.listen, Subcommand.tubelisten]:
-            player.extend([f"--input-ipc-server={args.mpv_socket}", "--no-video", "--keep-open=no", "--really-quiet"])
-        elif args.action in [Subcommand.watch, Subcommand.tubewatch]:
-            player.extend(["--fs", "--force-window=yes", "--really-quiet"])
-
-        if args.action in [Subcommand.tubelisten, Subcommand.tubewatch]:
-            player.extend(["--script-opts=ytdl_hook-try_ytdl_first=yes"])
-
-        start, end = calculate_duration(args, m)
-        if end == 0:
-            return
-        if start != 0:
-            player.extend([f"--start={int(start)}", "--no-save-position-on-quit"])
-        if end != m.duration:
-            player.extend([f"--end={int(end)}"])
-
-    if args.action == Subcommand.watch:
-        if m["subtitle_count"] > 0:
-            player.extend(args.player_args_when_sub)
-        else:
-            player.extend(args.player_args_when_no_sub)
-
-    if args.action in [Subcommand.watch, Subcommand.tubewatch]:
-        cmd(*player, "--", media_file, strict=not args.ignore_errors)
-    elif args.action in [Subcommand.listen, Subcommand.tubelisten]:
-        cmd_interactive(*player, "--", media_file)
-
-
 def chromecast_play(args, m):
-    if args.action in [Subcommand.watch, Subcommand.tubewatch]:
-        catt_log = watch_chromecast(args, m)
-    elif args.action in [Subcommand.listen, Subcommand.tubelisten]:
+    if args.action in [SC.watch, SC.tubewatch]:
+        catt_log = watch_chromecast(args, m, subtitles_file=externalize_subtitle(m["path"]))
+    elif args.action in [SC.listen, SC.tubelisten]:
         catt_log = listen_chromecast(args, m)
     else:
         raise NotImplementedError
@@ -538,10 +354,10 @@ def play(args, media: pd.DataFrame):
 
         if any(
             [
-                args.play_in_order > 1 and args.action not in [Subcommand.listen, Subcommand.tubelisten],
-                args.play_in_order >= 1 and args.action == Subcommand.listen and "audiobook" in media_file.lower(),
+                args.play_in_order > 1 and args.action not in [SC.listen, SC.tubelisten],
+                args.play_in_order >= 1 and args.action == SC.listen and "audiobook" in media_file.lower(),
                 args.play_in_order >= 1
-                and args.action == Subcommand.tubelisten
+                and args.action == SC.tubelisten
                 and m["title"]
                 and "audiobook" in m["title"].lower(),
             ]
@@ -558,9 +374,9 @@ def play(args, media: pd.DataFrame):
             if args.transcode:
                 media_file = transcode(media_file)
 
-        if args.action in [Subcommand.watch, Subcommand.tubewatch]:
+        if args.action in [SC.watch, SC.tubewatch]:
             print(media_file)
-        elif args.action == Subcommand.listen:
+        elif args.action == SC.listen:
             print(cmd("ffprobe", "-hide_banner", "-loglevel", "info", media_file).stderr)
 
         if args.chromecast:
@@ -576,7 +392,48 @@ def play(args, media: pd.DataFrame):
             post_act(args, media_file)
 
 
-def construct_query(args):
+audio_include_string = (
+    lambda x: f"""and (
+    path like :include{x}
+    OR mood like :include{x}
+    OR genre like :include{x}
+    OR year like :include{x}
+    OR bpm like :include{x}
+    OR key like :include{x}
+    OR time like :include{x}
+    OR decade like :include{x}
+    OR categories like :include{x}
+    OR city like :include{x}
+    OR country like :include{x}
+    OR description like :include{x}
+    OR album like :include{x}
+    OR title like :include{x}
+    OR artist like :include{x}
+)"""
+)
+
+audio_exclude_string = (
+    lambda x: f"""and (
+    path not like :exclude{x}
+    AND mood not like :exclude{x}
+    AND genre not like :exclude{x}
+    AND year not like :exclude{x}
+    AND bpm not like :exclude{x}
+    AND key not like :exclude{x}
+    AND time not like :exclude{x}
+    AND decade not like :exclude{x}
+    AND categories not like :exclude{x}
+    AND city not like :exclude{x}
+    AND country not like :exclude{x}
+    AND description not like :exclude{x}
+    AND album not like :exclude{x}
+    AND title not like :exclude{x}
+    AND artist not like :exclude{x}
+)"""
+)
+
+
+def construct_fs_query(args):
     cf = []
     bindings = {}
 
@@ -587,20 +444,13 @@ def construct_query(args):
 
     cf.extend([" and " + w for w in args.where])
 
-    if args.action == Subcommand.listen:
+    if args.action == SC.listen:
         for idx, inc in enumerate(args.include):
             cf.append(audio_include_string(idx))
             bindings[f"include{idx}"] = "%" + inc.replace(" ", "%").replace("%%", " ") + "%"
         for idx, exc in enumerate(args.exclude):
             cf.append(audio_exclude_string(idx))
             bindings[f"exclude{idx}"] = "%" + exc.replace(" ", "%").replace("%%", " ") + "%"
-    elif args.action in [Subcommand.tubewatch, Subcommand.tubelisten]:
-        for idx, inc in enumerate(args.include):
-            cf.append(tube_include_string(idx))
-            bindings[f"include{idx}"] = "%" + inc.replace(" ", "%").replace("%%", " ") + "%"
-        for idx, exc in enumerate(args.exclude):
-            cf.append(tube_exclude_string(idx))
-            bindings[f"exclude{idx}"] = "%" + exc.replace(" ", "%").replace("%%", " ") + "%"
     else:
         bindings = []
         for inc in args.include:
@@ -616,122 +466,30 @@ def construct_query(args):
     OFFSET = f"OFFSET {args.skip}" if args.skip else ""
 
     query = f"""SELECT path
-        {', duration' if args.action != Subcommand.filesystem else ''}
-        {', subtitle_count' if args.action == Subcommand.watch else ''}
         , size
-        {', ' + ', '.join(args.print_column) if args.print_column else ''}
-        {', sparseness' if args.action == Subcommand.filesystem else ''}
-        {', is_dir' if args.action == Subcommand.filesystem else ''}
-        {', title' if args.action in [Subcommand.tubelisten, Subcommand.tubewatch] else ''}
-        {', ' + ', '.join(args.print_column) if args.print_column else ''}
+        {', duration' if args.action in [SC.listen, SC.watch] else ''}
+        {', subtitle_count' if args.action == SC.watch else ''}
+        {', sparseness' if args.action == SC.filesystem else ''}
+        {', is_dir' if args.action == SC.filesystem else ''}
+        {', ' + ', '.join(args.cols) if args.cols else ''}
     FROM media
     WHERE 1=1
     {args.sql_filter}
-    {'and audio_count > 0' if args.action == Subcommand.listen else ''}
-    {'and video_count > 0' if args.action == Subcommand.watch else ''}
+    {'and audio_count > 0' if args.action == SC.listen else ''}
+    {'and video_count > 0' if args.action == SC.watch else ''}
     {'and path not like "%/keep/%"' if args.post_action == 'askkeep' else ''}
     {'and width < height' if args.portrait else ''}
     ORDER BY 1=1
         {',' + args.sort if args.sort else ''}
         {', path' if args.print or args.include or args.play_in_order > 0 else ''}
-        {', duration / size ASC' if args.action != Subcommand.filesystem else ''}
+        {', duration / size ASC' if args.action in [SC.listen, SC.watch] else ''}
     {LIMIT} {OFFSET}
     """
 
     return query, bindings
 
 
-def printer(args, query, bindings):
-    if "a" in args.print:
-        query = f"""select
-            "Aggregate" as path
-            {', sum(duration) duration' if args.action != Subcommand.filesystem else ''}
-            {', sparseness' if args.action == Subcommand.filesystem else ''}
-            , sum(size) size
-            , count(*) count
-            {', ' + ', '.join([f'sum({c}) sum_{c}' for c in args.print_column]) if args.print_column else ''}
-            {', ' + ', '.join([f'avg({c}) avg_{c}' for c in args.print_column]) if args.print_column else ''}
-        from ({query}) """
-
-    if args.print and "q" in args.print:
-        return print(query)
-
-    db_resp = pd.DataFrame([dict(r) for r in args.con.execute(query, bindings).fetchall()])
-    db_resp.dropna(axis="columns", how="all", inplace=True)
-
-    if args.verbose > 1 and args.print_column and "*" in args.print_column:
-        import rich
-
-        breakpoint()
-        for t in db_resp.to_dict(orient="records"):
-            rich.print(t)
-
-    if db_resp.empty or len(db_resp) == 0:
-        print("No media found")
-        exit(2)
-
-    if args.delete:
-        print(f"Deleting {len(db_resp)} metadata records")
-        return remove_media(args, db_resp[["path"]].values.tolist(), quiet=True)
-
-    if args.mark_watched:
-        print(f"{len(db_resp)} metadata records marked watched")
-        return mark_media_watched(args, db_resp[["path"]].values.tolist())
-
-    if "f" in args.print:
-        if args.limit == 1:
-            f = db_resp[["path"]].loc[0].iat[0]
-            if not Path(f).exists():
-                remove_media(args, f)
-                return printer(args, query, bindings)
-            print(f)
-        else:
-            if not args.print_column:
-                args.print_column = ["path"]
-
-            for line in db_resp[[*args.print_column]].to_string(index=False, header=False).splitlines():
-                print(line.strip())
-    else:
-        tbl = db_resp.copy()
-        resize_col(tbl, "path", 22)
-        resize_col(tbl, "title", 18)
-
-        if "size" in tbl.columns:
-            tbl[["size"]] = tbl[["size"]].applymap(lambda x: None if x is None else humanize.naturalsize(x))
-
-        if "duration" in tbl.columns:
-            tbl[["duration"]] = tbl[["duration"]].applymap(human_time)
-        resize_col(tbl, "duration", 6)
-
-        for t in ["time_modified", "time_created"]:
-            if t in tbl.columns:
-                tbl[[t]] = tbl[[t]].applymap(
-                    lambda x: None if x is None else humanize.naturaldate(datetime.fromtimestamp(x))
-                )
-
-        print(tabulate(tbl, tablefmt="fancy_grid", headers="keys", showindex=False))  # type: ignore
-
-        if args.action != Subcommand.filesystem:
-            if len(db_resp) > 1:
-                print(f"{len(db_resp)} items")
-            summary = db_resp.sum(numeric_only=True)
-            duration = summary.get("duration") or 0
-            duration = human_time(duration)
-            print("Total duration:", duration)
-
-
-def pkill(*command, strict=True):
-    found = 0
-    for process in psutil.process_iter():
-        if process.cmdline() == command:
-            process.terminate()
-            found = 1
-            break
-    if strict and found == 0:
-        raise Exception("Process not found")
-
-
-def process_actions(args):
+def process_actions(args, construct_query=construct_fs_query):
     args.con = sqlite_con(args.database)
     query, bindings = construct_query(args)
 
@@ -757,18 +515,15 @@ def process_actions(args):
 
 
 def watch():
-    args = parse_args("video.db", default_chromecast="Living Room TV")
-    args.action = Subcommand.watch
+    args = parse_args(SC.watch, "video.db", default_chromecast="Living Room TV")
     process_actions(args)
 
 
 def listen():
-    args = parse_args("audio.db", default_chromecast="Xylo and Orchestra")
-    args.action = Subcommand.listen
+    args = parse_args(SC.listen, "audio.db", default_chromecast="Xylo and Orchestra")
     process_actions(args)
 
 
 def filesystem():
-    args = parse_args("fs.db")
-    args.action = Subcommand.filesystem
+    args = parse_args(SC.filesystem, "fs.db")
     process_actions(args)
